@@ -65,6 +65,8 @@ sub new {
 
     $obj->{ID} ||= Yote::ObjProvider::get_id( $obj );
     $obj->_init() if $needs_init;
+
+    $obj->_load();
     
     if( ref( $id_or_hash ) eq 'HASH' ) {
 	for my $key ( %$id_or_hash ) {
@@ -141,6 +143,12 @@ sub _path_to_root {
 # These methods are not part of the public API
 #
 
+
+sub _allows_update {
+    my( $self, $field, $account ) = @_;
+    return $field =~ /^[A-Z]/ || ( $account && $account->get_login()->get__is_root() );
+}
+
 #
 # Converts scalar, yote object, hash or array to data for returning.
 #
@@ -155,6 +163,12 @@ sub __obj_to_response {
             if( $tied ) {
                 $d = $tied->[1];
                 $use_id = Yote::ObjProvider::get_id( $to_convert );
+		for my $entry (@$d) {
+		    next unless $entry;
+		    if( index( $entry, 'v' ) != 0 ) {
+			Yote::ObjManager::register_object( $entry, $login ? $login->{ID} : $guest_token );
+		    }
+		}
             } else {
                 $d = $self->__transform_data_no_id( $to_convert, $login, $guest_token );
             }
@@ -164,6 +178,12 @@ sub __obj_to_response {
             if( $tied ) {
                 $d = $tied->[1];
                 $use_id = Yote::ObjProvider::get_id( $to_convert );
+		for my $entry (values %$d) {
+		    next unless $entry;
+		    if( index( $entry, 'v' ) != 0 ) {
+			Yote::ObjManager::register_object( $entry, $login ? $login->{ID} : $guest_token );
+		    }
+		}
             } else {
                 $d = $self->__transform_data_no_id( $to_convert, $login, $guest_token );
             }
@@ -171,12 +191,16 @@ sub __obj_to_response {
         else {
             $use_id = Yote::ObjProvider::get_id( $to_convert );
             $d = { map { $_ => $to_convert->{DATA}{$_} } grep { $_ && $_ !~ /^_/ } keys %{$to_convert->{DATA}}};
-
+	    for my $vl (values %$d) {
+		if( index( $vl, 'v' ) != 0 ) {
+		    Yote::ObjManager::register_object( $vl, $login ? $login->{ID} : $guest_token );
+		}
+	    }
 	    $m = Yote::ObjProvider::package_methods( $ref );
         }
 
-	Yote::ObjManager::register_object( $use_id, $login, $guest_token ) if $use_id;
-	return $m ? { a => ref( $self ), c => $ref, id => $use_id, d => $d, 'm' => $m } : { a => ref( $self ), c => $ref, id => $use_id, d => $d };
+	Yote::ObjManager::register_object( $use_id, $login ? $login->{ID} : $guest_token ) if $use_id;
+	return $m ? { c => $ref, id => $use_id, d => $d, 'm' => $m } : { c => $ref, id => $use_id, d => $d };
     } # if a reference
     return "v$to_convert";
 } #__obj_to_response
@@ -189,20 +213,24 @@ sub __transform_data_no_id {
     if( ref( $item ) eq 'ARRAY' ) {
         my $tied = tied @$item;
         if( $tied ) {
-            return Yote::ObjProvider::get_id( $item ); 
+	    my $id =  Yote::ObjProvider::get_id( $item ); 
+	    Yote::ObjManager::register_object( $id, $login ? $login->{ID} : $guest_token );
+            return $id;
         }
         return [map { $self->__obj_to_response( $_, $login, $guest_token ) } @$item];
     }
     elsif( ref( $item ) eq 'HASH' ) {
         my $tied = tied %$item;
         if( $tied ) {
-            return Yote::ObjProvider::get_id( $item ); 
+	    my $id =  Yote::ObjProvider::get_id( $item ); 
+	    Yote::ObjManager::register_object( $id, $login ? $login->{ID} : $guest_token );
+            return $id;
         }
         return { map { $_ => $self->__obj_to_response( $item->{$_}, $login, $guest_token ) } keys %$item };
     }
     elsif( ref( $item ) ) {
         my $id = Yote::ObjProvider::get_id( $item ); 
-	Yote::ObjManager::register_object( $id, $login, $guest_token );
+	Yote::ObjManager::register_object( $id, $login ? $login->{ID} : $guest_token );
 	return $id;
     }
     else {
@@ -215,17 +243,6 @@ sub __transform_data_no_id {
 #      * PUBLIC METHODS *
 # ------------------------------------------------------------------------------------------
 
-sub load_direct_descendents {
-    my( $self, $data, $account ) = @_;
-    my @ret;
-    for my $fld (grep { $_ !~ /^_/ } keys %{$self->{DATA}}) {
-        my $val = $self->{DATA}{$fld};
-        next unless $val > 0;
-        push( @ret, Yote::ObjProvider::xform_out( $val ) );
-    }
-    return \@ret;
-} #load_direct_descendents 
-
 sub paginate {
     my( $self, $data, $account ) = @_;
     
@@ -235,6 +252,15 @@ sub paginate {
 
 } #paginate
 
+sub paginate_rev {
+    my( $self, $data, $account ) = @_;
+    
+    my( $list_name, $number, $start ) = @$data;
+
+    return Yote::ObjProvider::paginate_xpath_list( $self->_path_to_root() . "/$list_name", $number, $start, 1 );
+
+} #paginate_rev
+
 sub paginate_hash {
     my( $self, $data, $account ) = @_;
     my( $list_name, $number, $start ) = @$data;
@@ -247,10 +273,6 @@ sub paginate_hash {
 
 } #paginate_hash
 
-sub _allows_update {
-    my( $self, $field, $account ) = @_;
-    return $field =~ /^[A-Z]/ || ( $account && $account->get_login()->get__is_root() );
-}
 
 #
 # Updates the object but only for capitolized keys that already exist.
@@ -526,11 +548,6 @@ This method is called each time an object is loaded from the data store.
 
 =over 4
 
-=item load_direct_descendents
-
-This method takes no arguments and does not need an account to function. This method
-returns a list of objects that are directly attached to this one.
-
 =item paginate
 
 This method takes a list ref with three entries : [field_name, number of items to return, starting point]. 
