@@ -62,8 +62,8 @@ sub commit_transaction {
 sub dirty {
     my $obj = shift;
     my $id = shift;
+    Yote::ObjManager::mark_dirty( $id );
     $Yote::ObjProvider::DIRTY->{$id} = $obj;
-    $Yote::ObjProvider::CHANGED->{$id} = 1;
 } #dirty
 
 
@@ -234,8 +234,8 @@ sub paginate_xpath {
 #   rather than [ undef, undef, undef, 'val1', 'val2' ]
 #
 sub paginate_xpath_list {
-    my( $path, $paginate_length, $paginate_start ) = @_;
-    return [ map { xform_out( $_ ) } @{ $DATASTORE->paginate_xpath_list( $path, $paginate_length, $paginate_start ) } ];
+    my( $path, $paginate_length, $paginate_start, $reverse ) = @_;
+    return [ map { xform_out( $_ ) } @{ $DATASTORE->paginate_xpath_list( $path, $paginate_length, $paginate_start, $reverse ) } ];
 } #paginate_xpath_list
 
 sub paths_to_root {
@@ -320,9 +320,6 @@ sub recycle_objects {
     return $DATASTORE->recycle_objects( @_ );
 } #recycle_objects
 
-sub reset_changed {
-    $Yote::ObjProvider::CHANGED = {};
-}
 sub start_transaction {
     return $DATASTORE->start_transaction();
 }
@@ -459,10 +456,6 @@ sub __clean {
     delete $Yote::ObjProvider::DIRTY->{$id};
 } #__clean
 
-sub __fetch_changed {
-    return [keys %{$Yote::ObjProvider::CHANGED}];
-}
-
 sub __is_dirty {
     my $obj = shift;
     my $id = ref($obj) ? get_id($obj) : $obj;
