@@ -1,5 +1,7 @@
 package Yote::Util::MessageBoard;
 
+use strict;
+
 use Yote::Obj;
 
 use base 'Yote::Obj';
@@ -62,21 +64,21 @@ sub read {
     } #filters
     
     if( $data->{sort} eq 'date' ) {
-	if( $sort_asc ) {
+	if( $data->{sort_asc} ) {
 	    $all_messages = [ sort { $a->get_sent() <=> $b->get_sent()  } @$all_messages ];
 	} else {
 	    $all_messages = [ sort { $b->get_sent() <=> $a->get_sent()  } @$all_messages ];
 	}
     }
     elsif( $data->{sort} eq 'name' ) {
-	if( $sort_desc ) {
+	if( $data->{sort_desc} ) {
 	    $all_messages = [ sort { $b->get_from()->get_handle() cmp $a->get_from()->get_handle()  } @$all_messages ];
 	} else {
 	    $all_messages = [ sort { $a->get_from()->get_handle() cmp $b->get_from()->get_handle()  } @$all_messages ];
 	}	
     }
     elsif( $data->{sort} eq 'subject' ) {
-	if( $sort_desc ) {
+	if( $data->{sort_desc} ) {
 	    $all_messages = [ sort { $b->get_subject() cmp $a->get_subject()  } @$all_messages ];
 	} else {
 	    $all_messages = [ sort { $a->get_subject() cmp $b->get_subject()  } @$all_messages ];
